searchData={"content_type":"text/markdown","items":[{"doc":"Terminal UI framework for Elixir.\n\nA framework empowering developers to build complex terminal UI applications. Orange provides a declarative way to build UI components and manage terminal events. Orange offers stateful components which can encapsulate local state and logics, allows you to split your application into discrete sub-components. For example, here is a panel displays a random numbers every 1 second:\n    defmodule Panel do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: 0, events_subscription: true}\n\n      @impl true\n      def after_mount(_state, _attrs, update) do\n        spawn_link(fn -> random_number(update) end)\n      end\n\n      defp random_number(update) do\n        number = :rand.uniform(100)\n        update.(number)\n        Process.sleep(1000)\n        random_number(update)\n      end\n\n      @impl true\n      def handle_event(event, state, _attrs) do\n        case event do\n          %Orange.Terminal.KeyEvent{code: {:char, \"q\"}} ->\n            Orange.stop()\n            state\n\n          _ ->\n            state\n        end\n      end\n\n      @impl true\n      def render(state, _attrs, _update) do\n        rect style: [width: 25, height: 10, border: true] do\n          span do\n            \"Random number: #{state}\"\n          end\n        end\n      end\n    end\n\n    # Start the application. To quit, press 'q'.\n    Orange.start(Panel)","ref":"Orange.html","title":"Orange","type":"module"},{"doc":"Focus a component by component_id.\n\nThere can be only one focused component at a time. If a component is focused, it will receive all the events and prevent other components from receiving them. This is useful when you want a component to intercept all terminal events, for example, an input.","ref":"Orange.html#focus/1","title":"Orange.focus/1","type":"function"},{"doc":"Start the runtime and render the UI root.\n\nThe root component must be a custom component.","ref":"Orange.html#start/1","title":"Orange.start/1","type":"function"},{"doc":"Stop the runtime","ref":"Orange.html#stop/0","title":"Orange.stop/0","type":"function"},{"doc":"Subscribe to the event manager.\n\nSubscribed components will receive terminal events. To unsubscribe, use `unsubscribe/1`.","ref":"Orange.html#subscribe/1","title":"Orange.subscribe/1","type":"function"},{"doc":"defmodule Example do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: nil}\n\n      @impl true\n      def after_mount(_state, _attrs, _update) do\n        Orange.subscribe(:root)\n      end\n\n      @impl true\n      def render(_state, _attrs, _update) do\n        span do\n          \"Hello\"\n        end\n      end\n    end\n\n    Orange.start({Example, id: :root})","ref":"Orange.html#subscribe/1-examples","title":"Examples - Orange.subscribe/1","type":"function"},{"doc":"Unfocus a component by component_id.","ref":"Orange.html#unfocus/1","title":"Orange.unfocus/1","type":"function"},{"doc":"Unsubscribe to the event manager.","ref":"Orange.html#unsubscribe/1","title":"Orange.unsubscribe/1","type":"function"},{"doc":"A behaviour module for implementing custom components.\n\nA custom component is a self-contained UI component that can be reused across the application. Custom components are composed of other custom or primitive components.\n\nCustom components can have internal state and receive terminal events to update their state. They also have hooks, which will be trigger at specific points in the component lifecycle, for example, `after_mount/1` and `after_unmount/1`","ref":"Orange.Component.html","title":"Orange.Component","type":"behaviour"},{"doc":"In order to implement a custom component, you need to define a module that implements the `Orange.Component` behaviour. As an example, we will implement a Counter component. We can use up and down button to increase or decrease the counter value.\n\n    defmodule Counter do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: 0}\n\n      @impl true\n      def handle_event(event, state, _attrs) do\n        case event do\n          %Orange.Terminal.KeyEvent{code: :up} ->\n            state + 1\n\n          %Orange.Terminal.KeyEvent{code: :down} ->\n            state - 1\n\n          %Orange.Terminal.KeyEvent{code: {:char, \"q\"}} ->\n            Orange.stop()\n            state\n\n          _ ->\n            state\n        end\n      end\n\n      @impl true\n      def after_mount(state, _attrs, update) do\n        # Set counter value to 10 after 5 seconds\n        spawn(fn ->\n          Process.sleep(5000)\n          update.(10)\n        end)\n      end\n\n      @impl true\n      def render(state, attrs, _update) do\n        rect style: [width: 20, height: 20, border: attrs[:highlight]] do\n          span do\n            \"Counter: #{state}\"\n          end\n        end\n      end\n    end\n\nThis component can be used as children for other components like this:\n\n    defmodule App do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: nil}\n\n      def render do\n        rect style: [width: 20, height: 20, border: true] do\n          # No attributes\n          Counter\n\n          # Or with attributes\n          {Counter, id: :counter, highlight: true}\n        end\n      end\n    end","ref":"Orange.Component.html#module-examples","title":"Examples - Orange.Component","type":"behaviour"},{"doc":"There are two ways to update the component state:\n\n  1. By returning a new state from the `handle_event/3` callback\n\n  2. By using the update callback\n\nThe update callback will be passed to the `render/3` and lifecycle hooks. The update callback can be called with a new state or a function that receives the current state and returns the new state.\n\n    def render(state, attrs, update_callback)\n      update_callback.(%{state | counter: state.counter + 1})\n      update_callback.(fn state -> %{state | counter: state.counter - 1} end)\n    end","ref":"Orange.Component.html#module-update-callback","title":"Update callback - Orange.Component","type":"behaviour"},{"doc":"Lifecycle hook that is called after the component is mounted (first time render to the terminal).","ref":"Orange.Component.html#c:after_mount/3","title":"Orange.Component.after_mount/3","type":"callback"},{"doc":"Lifecycle hook that is called after the component is unmounted (remove from the terminal).","ref":"Orange.Component.html#c:after_unmount/3","title":"Orange.Component.after_unmount/3","type":"callback"},{"doc":"Receives the event, current state, and attributes and returns the new state.","ref":"Orange.Component.html#c:handle_event/3","title":"Orange.Component.handle_event/3","type":"callback"},{"doc":"Receives the attributes map from parent component and returns the component configuration.\n\nConfiguration keys are:\n\n  * `state` - The initial state of the component\n\n  * `events_subscription` - whether the component should subscribe to terminal events","ref":"Orange.Component.html#c:init/1","title":"Orange.Component.init/1","type":"callback"},{"doc":"Receives the current state, attributes and [update callback](#module-update-callback) and returns the rendered UI element.\n\n> ##","ref":"Orange.Component.html#c:render/3","title":"Orange.Component.render/3","type":"callback"},{"doc":">\n> This function must either return a single component or nil.","ref":"Orange.Component.html#c:render/3-info-info","title":"Info {: .info} - Orange.Component.render/3","type":"callback"},{"doc":"","ref":"Orange.Component.html#t:event/0","title":"Orange.Component.event/0","type":"type"},{"doc":"","ref":"Orange.Component.html#t:state/0","title":"Orange.Component.state/0","type":"type"},{"doc":"","ref":"Orange.Component.html#t:ui_element/0","title":"Orange.Component.ui_element/0","type":"type"},{"doc":"","ref":"Orange.Component.html#t:update_callback/0","title":"Orange.Component.update_callback/0","type":"type"},{"doc":"Macros for creating Orange primitive components\n\nCurrently, Orange supports three primitive components:\n  * `rect` for creating a rectangle\n  * `line` for creating a line\n  * `span` for creating a span","ref":"Orange.Macro.html","title":"Orange.Macro","type":"module"},{"doc":"Macro provides an ergonomic way to create Orange components. For example, the following code:\n\n    rect style: [width: 12, border: true], direction: :row do\n      span style: [color: :red] do\n        \"Hello\"\n      end\n\n      span do\n        \"World\"\n      end\n    end\n\nwill produce the following struct:\n\n    %Orange.Rect{\n      children: [\n        %Orange.Line{\n          children: [\n            %Orange.Span{children: [\"Hello\"], attributes: [style: [color: :red]]}\n          ],\n          attributes: []\n        },\n        %Orange.Line{\n          children: [%Orange.Span{children: [\"World\"], attributes: []}],\n          attributes: []\n        }\n      ],\n      attributes: [style: [width: 12, border: true], direction: :row]\n    }","ref":"Orange.Macro.html#module-examples","title":"Examples - Orange.Macro","type":"module"},{"doc":"Orange components can have children, provided by the do block. The syntax took inspiration from HTML:\n\n    rect do\n      span do\n        \"Hello\"\n      end\n\n      span do\n        \"World\"\n      end\n    end\n\nThe children can also be a list of components. This is useful when you want to render a collection of components. For example:\n\n    rect do\n      [\n        span do\n          \"Hello\"\n        end,\n\n        span do\n          \"World\"\n        end\n      ]\n    end\n\nNormal Elixir expression works inside the children block. For example, here's how to conditionally render components:\n\n    rect do\n      if :rand.uniform() > 0.5 do\n        \"Hello\"\n      else\n        \"World\"\n      end\n    end","ref":"Orange.Macro.html#module-components-children","title":"Components children - Orange.Macro","type":"module"},{"doc":"#","ref":"Orange.Macro.html#module-styling","title":"Styling - Orange.Macro","type":"module"},{"doc":"* `:width` - the width of the component\n\n  * `:height` - the height of the component\n\nThe values for `width` and `height` can be:\n\n  * An integer - the number of cells in the terminal. For example: `style: [width: 10]`\n\n  * A percentage string - the size is equal to the given percentage of the parent size. For example: `style: [width: \"50%\"]`\n\n  * An calc expression - the size is calculated based on the given expression. For example: `style: [width: \"calc(100% - 10)\"]`\n\n  * A fraction string - the size is calculated based on the given fraction of the parent size. All sibling components MUST also have a fraction size. The size will be calculated based on the component fraction divided by the sum of all sibling fractions. For example:\n\n    ```\n    rect style: [width: 12] do\n      rect style: [width: \"1fr\"] do\n      end\n\n      rect style: [width: \"2fr\"] do\n      end\n    end\n    ```\n\n    In this example, the first rect will have a width of 1/3 of the parent rect, and the second rect will have a width of 2/3 of the parent rect.\n\n#","ref":"Orange.Macro.html#module-sizing","title":"Sizing - Orange.Macro","type":"module"},{"doc":"Padding for the render box's inner content. The values for `padding` can be:\n\n  * One integer - padding for all sides. For example: `style: [padding: 1]` means `[padding_top: 1, padding_bottom: 1, padding_left: 1, padding_right: 1]`\n\n  * Two integers tuple - padding vertical and padding horizontal. For example: `style: [padding: {1, 2}]` means `[padding_top: 1, padding_bottom: 1, padding_left: 2, padding_right: 2]`\n\n  * Four integers tuple - padding top, right, bottom, left respectively. For example: `style: [padding: {1, 2, 3, 4}]` means `[padding_top: 1, padding_bottom: 3, padding_left: 4, padding_right: 2]`\n\n#","ref":"Orange.Macro.html#module-padding","title":"Padding - Orange.Macro","type":"module"},{"doc":"* `:border` - whether to render a border around the rect. Defaults to `false`\n\n  * `:border_top` - whether to render a top border. Defaults to `true` if `border` is `true`\n\n  * `:border_bottom` - whether to render a bottom border. Defaults to `true` if `border` is `true`\n\n  * `:border_left` - whether to render a left border. Defaults to `true` if `border` is `true`\n\n  * `:border_right` - whether to render a right border. Defaults to `true` if `border` is `true`\n\n  * `:border_color` - the color of the border. See [Color](#module-color) section for supported colors\n\n#","ref":"Orange.Macro.html#module-border","title":"Border - Orange.Macro","type":"module"},{"doc":"* `:color` - the color of the component text. The color value can be inherited from the parent component. If the color value is not specified, the component will inherit the color from the parent component\n\n  * `:background_color` - the color of the component background\n\nThe values for `:color` and `:background_color` is a single atom representing the color. Supported colors are:\n\n  * `:white`\n  * `:black`\n  * `:grey`\n  * `:dark_grey`\n  * `:red`\n  * `:dark_red`\n  * `:green`\n  * `:dark_green`\n  * `:yellow`\n  * `:dark_yellow`\n  * `:blue`\n  * `:dark_blue`\n  * `:magenta`\n  * `:dark_magenta`\n  * `:cyan`\n  * `:dark_cyan`","ref":"Orange.Macro.html#module-color","title":"Color - Orange.Macro","type":"module"},{"doc":"Generates a `Orange.Line` struct","ref":"Orange.Macro.html#line/2","title":"Orange.Macro.line/2","type":"macro"},{"doc":"* `:style` - style attributes for the line. Supported keys are:\n\n    * `:width` - see [Sizing](#module-sizing) section\n\n    * `:height` - see [Sizing](#module-sizing) section\n\n    * `:padding` - see [Padding](#module-padding) section\n\n    * `:color` - see [Color](#module-color) section\n\n    * `:background_color` - see [Color](#module-color) section","ref":"Orange.Macro.html#line/2-options","title":"Options - Orange.Macro.line/2","type":"macro"},{"doc":"Line only accepts `Orange.Span` as children. To provide better ergonomics, the macro will automatically wrap string to `Orange.Span`. For example, the following code:\n\n    line do\n      \"Hello\"\n    end\n\n  will produce the result:\n\n    %Orange.Line{\n      children: [%Orange.Span{children: [\"Hello\"], attributes: []}],\n      attributes: []\n    }","ref":"Orange.Macro.html#line/2-children-validation","title":"Children validation - Orange.Macro.line/2","type":"macro"},{"doc":"iex> import Orange.Macro\n    iex> line style: [width: 5, color: :red] do\n    ...>   span do\n    ...>     \"foo\"\n    ...>   end\n    ...> end\n    %Orange.Line{\n      children: [%Orange.Span{children: [\"foo\"], attributes: []}],\n      attributes: [style: [width: 5, color: :red]]\n    }","ref":"Orange.Macro.html#line/2-examples","title":"Examples - Orange.Macro.line/2","type":"macro"},{"doc":"Generates a `Orange.Rect` struct","ref":"Orange.Macro.html#rect/2","title":"Orange.Macro.rect/2","type":"macro"},{"doc":"* `:direction` - the direction to layout children `:row` or `:column` (default)\n\n  * `:style` - style attributes for the rect. Supported keys are:\n\n    * `:width` - see [Sizing](#module-sizing) section\n\n    * `:height` - see [Sizing](#module-sizing) section\n\n    * `:border` - see [Border](#module-border) section\n\n    * `:padding` - see [Padding](#module-padding) section\n\n    * `:color` - see [Color](#module-color) section\n\n    * `:background_color` - see [Color](#module-color) section\n\n  * `:title` - the title of the rect. If specified, it implies `border` is `true`\n\n  * `:scroll_x` - the horizontal scroll offset\n\n  * `:scroll_y` - the vertical scroll offset","ref":"Orange.Macro.html#rect/2-options","title":"Options - Orange.Macro.rect/2","type":"macro"},{"doc":"Rect only accepts `Orange.Rect` and `Orange.Line` as children. To provide better ergonomics, the macro will automatically wrap `Orange.Span` and string to `Orange.Line`. For example, the following code:\n\n    rect do\n      \"Hello\"\n    end\n\n    rect do\n      span do\n        \"Hello\"\n      end\n    end\n\n  will both produce the result:\n\n    %Orange.Rect{\n      children: [\n        %Orange.Line{\n          children: [%Orange.Span{children: [\"Hello\"], attributes: []}],\n          attributes: []\n        }\n      ],\n      attributes: []\n    }","ref":"Orange.Macro.html#rect/2-children-validation","title":"Children validation - Orange.Macro.rect/2","type":"macro"},{"doc":"iex> import Orange.Macro\n    iex> rect style: [width: 5, border: true], direction: :row do\n    ...>   \"foo\"\n    ...>\n    ...>   span do\n    ...>     \"bar\"\n    ...>   end\n    ...> end\n    %Orange.Rect{\n      children: [\n        %Orange.Line{\n          children: [%Orange.Span{children: [\"foo\"], attributes: []}],\n          attributes: []\n        },\n        %Orange.Line{\n          children: [%Orange.Span{children: [\"bar\"], attributes: []}],\n          attributes: []\n        }\n      ],\n      attributes: [style: [width: 5, border: true], direction: :row]\n    }","ref":"Orange.Macro.html#rect/2-examples","title":"Examples - Orange.Macro.rect/2","type":"macro"},{"doc":"Generates a `Orange.Span` struct","ref":"Orange.Macro.html#span/2","title":"Orange.Macro.span/2","type":"macro"},{"doc":"* `:style` - style attributes for the line. Supported keys are:\n\n    * `:color` - see [Color](#module-color) section\n\n    * `:background_color` - see [Color](#module-color) section","ref":"Orange.Macro.html#span/2-options","title":"Options - Orange.Macro.span/2","type":"macro"},{"doc":"Span only accepts a single string as children","ref":"Orange.Macro.html#span/2-children-validation","title":"Children validation - Orange.Macro.span/2","type":"macro"},{"doc":"iex> import Orange.Macro\n    iex> span style: [color: :red] do\n    ...>   \"foo\"\n    ...> end\n    %Orange.Span{children: [\"foo\"], attributes: [style: [color: :red]]}","ref":"Orange.Macro.html#span/2-examples","title":"Examples - Orange.Macro.span/2","type":"macro"},{"doc":"The runtime module is responsible for rendering the UI components. The runtime consists of multiple\ncomponents:\n\n1. `Orange.Runtime.RenderLoop`: UI rendering loop, implemented as a GenServer. Re-render is triggered in these cases:\n  a. Receive events from the event poller\n  b. Receive state update requests from update callbacks\n2. `Orange.Runtime.EventManager`: manages event subscriptions and dispatches events to the subscribed components\n3. `Orange.Runtime.EventPoller`: polls for events from the terminal and sends to the render loop","ref":"Orange.Runtime.html","title":"Orange.Runtime","type":"module"},{"doc":"","ref":"Orange.Runtime.html#focus/1","title":"Orange.Runtime.focus/1","type":"function"},{"doc":"","ref":"Orange.Runtime.html#start/1","title":"Orange.Runtime.start/1","type":"function"},{"doc":"","ref":"Orange.Runtime.html#stop/0","title":"Orange.Runtime.stop/0","type":"function"},{"doc":"","ref":"Orange.Runtime.html#subscribe/1","title":"Orange.Runtime.subscribe/1","type":"function"},{"doc":"","ref":"Orange.Runtime.html#unfocus/1","title":"Orange.Runtime.unfocus/1","type":"function"},{"doc":"","ref":"Orange.Runtime.html#unsubscribe/1","title":"Orange.Runtime.unsubscribe/1","type":"function"},{"doc":"Render loop process, holds the state of the UI and triggers re-renders.","ref":"Orange.Runtime.RenderLoop.html","title":"Orange.Runtime.RenderLoop","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Orange.Runtime.RenderLoop.html#child_spec/1","title":"Orange.Runtime.RenderLoop.child_spec/1","type":"function"},{"doc":"Given a component_id, returns the component ref","ref":"Orange.Runtime.RenderLoop.html#component_ref_by_id/1","title":"Orange.Runtime.RenderLoop.component_ref_by_id/1","type":"function"},{"doc":"Provides API to interact with the terminal.","ref":"Orange.Terminal.html","title":"Orange.Terminal","type":"behaviour"},{"doc":"Disable terminal raw mode.","ref":"Orange.Terminal.html#disable_raw_mode/0","title":"Orange.Terminal.disable_raw_mode/0","type":"function"},{"doc":"","ref":"Orange.Terminal.html#c:disable_raw_mode/0","title":"Orange.Terminal.disable_raw_mode/0","type":"callback"},{"doc":"","ref":"Orange.Terminal.html#c:draw/1","title":"Orange.Terminal.draw/1","type":"callback"},{"doc":"Draws the buffer to the terminal. If a previous buffer is provided, it will only draw the diff between the two buffers.","ref":"Orange.Terminal.html#draw/2","title":"Orange.Terminal.draw/2","type":"function"},{"doc":"","ref":"Orange.Terminal.html#c:draw/2","title":"Orange.Terminal.draw/2","type":"callback"},{"doc":"Enable terminal raw mode.","ref":"Orange.Terminal.html#enable_raw_mode/0","title":"Orange.Terminal.enable_raw_mode/0","type":"function"},{"doc":"","ref":"Orange.Terminal.html#c:enable_raw_mode/0","title":"Orange.Terminal.enable_raw_mode/0","type":"callback"},{"doc":"Enter terminal alternate screen.","ref":"Orange.Terminal.html#enter_alternate_screen/0","title":"Orange.Terminal.enter_alternate_screen/0","type":"function"},{"doc":"","ref":"Orange.Terminal.html#c:enter_alternate_screen/0","title":"Orange.Terminal.enter_alternate_screen/0","type":"callback"},{"doc":"Hide terminal cursor.","ref":"Orange.Terminal.html#hide_cursor/0","title":"Orange.Terminal.hide_cursor/0","type":"function"},{"doc":"","ref":"Orange.Terminal.html#c:hide_cursor/0","title":"Orange.Terminal.hide_cursor/0","type":"callback"},{"doc":"Leave terminal alternate screen.","ref":"Orange.Terminal.html#leave_alternate_screen/0","title":"Orange.Terminal.leave_alternate_screen/0","type":"function"},{"doc":"","ref":"Orange.Terminal.html#c:leave_alternate_screen/0","title":"Orange.Terminal.leave_alternate_screen/0","type":"callback"},{"doc":"Polls the terminal for events.","ref":"Orange.Terminal.html#poll_event/0","title":"Orange.Terminal.poll_event/0","type":"function"},{"doc":"","ref":"Orange.Terminal.html#c:poll_event/0","title":"Orange.Terminal.poll_event/0","type":"callback"},{"doc":"Show terminal cursor.","ref":"Orange.Terminal.html#show_cursor/0","title":"Orange.Terminal.show_cursor/0","type":"function"},{"doc":"","ref":"Orange.Terminal.html#c:show_cursor/0","title":"Orange.Terminal.show_cursor/0","type":"callback"},{"doc":"","ref":"Orange.Terminal.html#terminal_size/0","title":"Orange.Terminal.terminal_size/0","type":"function"},{"doc":"","ref":"Orange.Terminal.html#c:terminal_size/0","title":"Orange.Terminal.terminal_size/0","type":"callback"}],"producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}